__title__ = 'admin_timeline.templatetags.admin_timeline_tags'
__version__ = '1.0'
__build__ = 0x00000a
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__all__ = ('assign', 'get_full_name')

from django import template

register = template.Library()

class AssignNode(template.Node):
    """
    Node for ``assign`` tag.
    """
    def __init__(self, value, as_var):
        self.value = value
        self.as_var = as_var

    def render(self, context):
        context[self.as_var] = self.value.resolve(context, True)
        return ''

@register.tag
def assign(parser, token):
    """
    Assign an expression to a variable in the current context.

    Syntax::
        {% assign [value] as [name] %}
    Example::
        {% assign entry.get_related as list %}
    """
    bits = token.contents.split()
    if len(bits) != 4:
        raise template.TemplateSyntaxError("'%s' tag takes three arguments" % bits[0])
    value = parser.compile_filter(bits[1])
    return AssignNode(value=value, as_var=bits[-1])


class GetFullNameNode(template.Node):
    """
    Node for ``get_full_name`` tag.
    """
    def __init__(self, user, as_var):
        self.user = user
        self.as_var = as_var

    def render(self, context):
        user = self.user.resolve(context, True)
        context[self.as_var] = user.get_full_name() or user.username
        return ''

@register.tag
def get_full_name(parser, token):
    """
    Get users' full name.

    Syntax::
        {% get_full_name [user] as [name] %}
    Example::
        {% get_full_name entry.user as user_full_name %}
    """
    bits = token.contents.split()
    if len(bits) != 4:
        raise template.TemplateSyntaxError("'%s' tag takes three arguments" % bits[0])
    user = parser.compile_filter(bits[1])
    return GetFullNameNode(user=user, as_var=bits[-1])



__title__ = 'admin_timeline.forms'
__version__ = '1.1'
__build__ = 0x00000b
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__all__ = ('FilterForm',)

from django import forms
from django.contrib.admin.models import LogEntry

# Getting a plain list of all users who have done anything.
data = LogEntry.objects.all().select_related('user', 'content_type').only('content_type', 'user')[:]

users = lambda data: list(set([(d.user_id, d.user.get_full_name() if d.user.get_full_name() else d.user.username) \
                               for d in data]))
content_types = lambda data: list(set([(d.content_type_id, d.content_type.name) for d in data]))

class FilterForm(forms.Form):
    """
    Filter form to be used in the timeline.

    ``users``: Users list to be filtered on.

    ``content_types``: Content types to be filtered on.
    """
    def __init__(self, *args, **kwargs):
        super(FilterForm, self).__init__(*args, **kwargs)
        self.fields['users'] = forms.MultipleChoiceField(choices=users(data), widget=forms.CheckboxSelectMultiple, \
                                                         required=False)
        self.fields['content_types'] = forms.MultipleChoiceField(choices=content_types(data), \
                                                                 widget=forms.CheckboxSelectMultiple, required=False)



__title__ = 'admin_timeline.views'
__version__ = '1.1'
__build__ = 0x00000b
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__all__ = ('log',)

import datetime

from django.http import HttpResponse
from django.utils import simplejson as json
from django.views.decorators.cache import never_cache
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.admin.models import LogEntry
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.template.loader import render_to_string
from django.template.defaultfilters import date as date_format
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.csrf import csrf_exempt

from admin_timeline.settings import NUMBER_OF_ENTRIES_PER_PAGE, SINGLE_LOG_ENTRY_DATE_FORMAT
from admin_timeline.settings import LOG_ENTRIES_DAY_HEADINGS_DATE_FORMAT
from admin_timeline.forms import FilterForm

@csrf_exempt
@never_cache
@staff_member_required
def log(request, template_name="admin_timeline/timeline.html", template_name_ajax="admin_timeline/timeline_ajax.html"):
    """
    Get number of log entires. Serves both non-AJAX and AJAX driven requests.

    Since we have a breakdown of entries per day per entry and we have an AJAX driven infinite scroll and we want to
    avoid having duplicated date headers, we always pass a variable named "last_date" when making another request
    to our main AJAX-driven view. So... this is our case scenario:

    Initial timeline rendered as a normal HTML (non AJAX request) (from a list of log entries). We send date of
    last element as "last_date" to the context too, which will be used an an initial value for a global JavaScript
    variable. Later on that date will be used to send it to the AJAX driven view and used in
    rendering ("render_to_string" method). After we have rendered the HTML to send back, we get the last date of
    the last element and send it along with the HTML rendered to our view in JSON response. When receiving the
    JSON response, we update the above mentioned global JavaScript variable with the value given.

    :param request: django.http.HttpRequest
    :param template_name: str
    :param template_name_ajax: str
    :return: django.http.HttpResponse

    This view accepts the following POST variables (all optional).
    :param page: int - Page number to get.
    :param user_id: int - If set, used to filter the user by.
    :param last_date: str - Example value "2012-05-24".
    :param start_date: str - If set, used as a start date to filter the actions with. Example value "2012-05-24".
    :param end_date: str - If set, used as an end date to filter the actions with. Example value "2012-05-24".

    NOTE: If it gets too complicatd with filtering, we need to have forms to validate and process the POST data.
    """
    def _get_date_from_string(s):
        """
        Gets date from a string given.

        :param s: str - date in string format
        :return: datetime.datetime
        """
        try:
            return datetime.date(*map(lambda x: int(x), s.split("-")))
        except Exception as e:
            return ""

    try:
        page = int(request.POST.get('page', 1))
        if page < 1:
            page = 1
    except Exception as e:
        page = 1

    users = []
    content_types = []
    filter_form = None

    if 'POST' == request.method:
        post = dict(request.POST)
        if 'users[]' in post:
            post['users'] = post.pop('users[]')
        if 'content_types[]' in post:
            post['content_types'] = post.pop('content_types[]')

        filter_form = FilterForm(post)
        if filter_form.is_valid():
            users = filter_form.cleaned_data['users']
            content_types = filter_form.cleaned_data['content_types']
        else:
            pass # Anything to do here?
    else:
        filter_form = FilterForm()

    # Some kind of a pagination
    start = (page - 1) * NUMBER_OF_ENTRIES_PER_PAGE
    end = page * NUMBER_OF_ENTRIES_PER_PAGE

    # Getting admin log entires taking page number into consideration.
    log_entries = LogEntry.objects.all().select_related('content_type', 'user')

    start_date = _get_date_from_string(request.POST.get('start_date'))
    end_date = _get_date_from_string(request.POST.get('end_date'))

    if start_date:
        log_entries = log_entries.filter(action_time__gte=start_date) # TODO

    if end_date:
        log_entries = log_entries.filter(action_time__lte=end_date) # TODO

    # If users given, filtering by users
    if users:
        log_entries = log_entries.filter(user__id__in=users)

    # If content types given, filtering by content types
    if content_types:
        log_entries = log_entries.filter(content_type__id__in=content_types)

    # Applying limits / freezing the queryset
    log_entries = log_entries[start:end]

    if log_entries:
        last_date = date_format(log_entries[len(log_entries) - 1].action_time, "Y-m-d")
    else:
        last_date = request.POST.get('last_date', None)

    # Using different template for AJAX driven requests
    if request.is_ajax():
        # Context to render the AJAX driven HTML with
        context = {
            'admin_log': log_entries,
            'number_of_entries_per_page': NUMBER_OF_ENTRIES_PER_PAGE,
            'page': page,
            'last_date': request.POST.get('last_date', None),
            'SINGLE_LOG_ENTRY_DATE_FORMAT': SINGLE_LOG_ENTRY_DATE_FORMAT,
            'LOG_ENTRIES_DAY_HEADINGS_DATE_FORMAT': LOG_ENTRIES_DAY_HEADINGS_DATE_FORMAT
        }

        # Rendering HTML for an AJAX driven request
        html = render_to_string(
            template_name_ajax,
            context,
            context_instance=RequestContext(request)
        )

        # Context to send back to user in a JSON response
        context = {
            'html': html,
            'last_date': last_date,
            'success': 1 if len(log_entries) else 0
        }
        return HttpResponse(json.dumps(context))

    # Context for a non-AJAX request
    context = {
        'admin_log': log_entries,
        'number_of_entries_per_page': NUMBER_OF_ENTRIES_PER_PAGE,
        'page': page,
        'last_date': last_date,
        'start_date': date_format(start_date, "Y-m-d") if start_date else "",
        'end_date': date_format(end_date, "Y-m-d") if end_date else "",
        'users': [int(u) for u in users],
        'content_types': [int(ct) for ct in content_types],
        'filter_form': filter_form,
        'SINGLE_LOG_ENTRY_DATE_FORMAT': SINGLE_LOG_ENTRY_DATE_FORMAT,
        'LOG_ENTRIES_DAY_HEADINGS_DATE_FORMAT': LOG_ENTRIES_DAY_HEADINGS_DATE_FORMAT,
        'title': _("Timeline") # For template breadcrumbs, etc.
    }

    return render_to_response(template_name, context, context_instance=RequestContext(request))



__title__ = 'admin_timeline.conf'
__version__ = '1.1'
__build__ = 0x00000b
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__all__ = ('get_setting',)

from django.conf import settings

from admin_timeline import defaults

def get_setting(setting, override=None):
    """
    Get a setting from ``admin_timeline`` conf module, falling back to the default.

    If override is not None, it will be used instead of the setting.

    :param setting: String with setting name
    :param override: Value to use when no setting is available. Defaults to None.
    :return: Setting value.
    """
    if override is not None:
        return override
    if hasattr(settings, 'ADMIN_TIMELINE_%s' % setting):
        return getattr(settings, 'ADMIN_TIMELINE_%s' % setting)
    else:
        return getattr(defaults, setting)



"""
Override the following values in your global ``settings`` module by adding `ADMIN_TIMELINE_` prefix to the values.
When it comes to importing the values, import them from ``admin_timeline.settings`` module (without the
`ADMIN_TIMELINE_` prefix).

``NUMBER_OF_ENTRIES_PER_PAGE``: Number of entries per page.

``SINGLE_LOG_ENTRY_DATE_FORMAT``: Date format for the single log entry. Default value is "g:i:s A".

``LOG_ENTRIES_DAY_HEADINGS_DATE_FORMAT``: Day headings date format. Default value is "l j F Y".

``DEBUG``
"""

__title__ = 'admin_timeline.settings'
__version__ = '1.1'
__build__ = 0x00000b
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__all__ = ('NUMBER_OF_ENTRIES_PER_PAGE', 'SINGLE_LOG_ENTRY_DATE_FORMAT', 'LOG_ENTRIES_DAY_HEADINGS_DATE_FORMAT', \
           'DEBUG')

from admin_timeline.conf import get_setting

NUMBER_OF_ENTRIES_PER_PAGE = get_setting('NUMBER_OF_ENTRIES_PER_PAGE')
SINGLE_LOG_ENTRY_DATE_FORMAT = get_setting('SINGLE_LOG_ENTRY_DATE_FORMAT')
LOG_ENTRIES_DAY_HEADINGS_DATE_FORMAT = get_setting('LOG_ENTRIES_DAY_HEADINGS_DATE_FORMAT')

DEBUG = get_setting('DEBUG')



__title__ = 'admin_timeline.defaults'
__version__ = '1.1'
__build__ = 0x00000b
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__all__ = ('NUMBER_OF_ENTRIES_PER_PAGE', 'LOG_ENTRIES_DAY_HEADINGS_DATE_FORMAT', 'SINGLE_LOG_ENTRY_DATE_FORMAT', \
           'DEBUG')

NUMBER_OF_ENTRIES_PER_PAGE = 35 # Number of entries per page. Used in both non-AJAX and AJAX driven views.

LOG_ENTRIES_DAY_HEADINGS_DATE_FORMAT = "l j F Y" # Headings (per day basis) date format in timeline.
SINGLE_LOG_ENTRY_DATE_FORMAT = "g:i:s A" # Single log entry date format in timeline.

DEBUG = False # Personal debug mode, which has nothing to do with global settings.DEBUG


import os
from setuptools import setup, find_packages

try:
    readme = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
except:
    readme = ''

template_dir = "src/admin_timeline/templates/admin_timeline"
templates = [os.path.join(template_dir, f) for f in os.listdir(template_dir)]

static_dir = "src/admin_timeline/static"
static_files = [os.path.join(static_dir, f) for f in os.listdir(static_dir)]

version = '1.1'

setup(
    name = 'django-admin-timeline',
    version = version,
    description = ("Facebook-like timeline for Django admin"),
    long_description = readme,
    classifiers = [
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "License :: OSI Approved :: GNU Lesser General Public License v2 or later (LGPLv2+)",
    ],
    keywords = 'django-admin-timeline, django, app, python',
    author = 'Artur Barseghyan',
    author_email = 'artur.barseghyan@gmail.com',
    url = 'https://bitbucket.org/barseghyanartur/django-admin-timeline',
    package_dir = {'':'src'},
    packages = find_packages(where='./src'),
    license = 'GPL 2.0/LGPL 2.1',
    package_data = {
        'admin_timeline': templates + static_files
    },
    include_package_data = True,
    install_requires = [
        'six==1.4.1',
        'radar==0.2',
    ]
)
